/*=========================================================================
                                                                                
  Program:   crea
  Module:    $RCSfile: creaException.h,v $
  Language:  C++
  Date:      $Date: 2009/06/04 14:24:07 $
  Version:   $Revision: 1.2 $
                                                                                
  Copyright (c) CREATIS (Centre de Recherche et d'Applications en Traitement de
  l'Image). All rights reserved. See Doc/License.txt or
  http://www.creatis.insa-lyon.fr/Public/crea/License.html for details.
                                                                                
     This software is distributed WITHOUT ANY WARRANTY; without even
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
     PURPOSE.  See the above copyright notices for more information.
                                                                                
=========================================================================*/


/**
 *  \file 
 *  \brief  class Exception:generic class for throwing any exception (header) 
 *
 *    Long description:
 */

/**
 *  \class crea::Exception 
 *  \brief  class Exception : generic class for throwing any exception 
 *
 *    Long description:
 */
 
#ifndef __creaException_h__
#define __creaException_h__

#include "creaSystem.h"
#include "creaMessageManager.h"
#include <exception>

namespace crea
{
  inline std::string bbGetObjectDescription() { return(""); }

  class CREA_EXPORT Exception : public std::exception
  {
  public:
    Exception(const std::string& object,
	      const std::string& source_file,
	      const std::string& message) throw()
      : mObject(object),
	mSourceFile(source_file),
	mMessage(message)
    {
       mWhatMessage = " * ERROR  : " + mMessage 
                    + " * OBJECT : " + mObject
                    + " * FILE "     + mSourceFile;
    }
    ~Exception() throw() {}
   virtual const char* what() const throw() 
   {
      return mWhatMessage.c_str();
   }

    void Print() throw()
    {
      std::cerr << "* ERROR  : " << mMessage <<std::endl; 
      int lev = crea::MessageManager::GetMessageLevel("Error");
      if (lev > 0) {
	std::cerr << "* OBJECT : " <<mObject<<std::endl;
	std::cerr << "* FILE   : " <<mSourceFile<<std::endl;
      }
    }
    const std::string& GetObject()     const { return mObject;     }
    const std::string& GetSourceFile() const { return mSourceFile; }
    const std::string& GetMessage()    const { return mMessage;    }
  private:
    std::string mObject;
    std::string mSourceFile;
    std::string mMessage;
    std::string mWhatMessage;
  };

}//namespace

#endif
