#include "creaRTTI.h"
#include "creaMessageManager.h"


 
namespace crea
{
  // Adapted from https://savannah.cern.ch/support/download.php?file_id=1972
  
  // Something to which we can cast the void*s, to allow finding the
  // dynamic typeid.
  struct Dummy {
    virtual ~Dummy() {}
  };
  

 void*  run_time_up_or_down_cast( const std::type_info& target_type,
			       const std::type_info& source_type,
			       const void*  source_pointer
			       )
 {
   return run_time_up_or_down_cast(target_type,
				   source_type,
				   const_cast<void*>(source_pointer));
 }

  void*   run_time_up_or_down_cast( const std::type_info& target_type,
			       const std::type_info& source_type,
			       void*  source_pointer
			       )
  {
    creaDebugMessage("info",5,
		     "run_time_up_or_down_cast : Casting pointer to '" 
		     << TypeName(typeid(*(Dummy*)source_pointer)) 
		     << "' from " << TypeName(source_type) 
		     << " to " << TypeName(target_type) << std::endl);

    void* target_pointer = 0;
#if __GNUC__ > 3 ||					\
  (__GNUC__ == 3 && (__GNUC_MINOR__ > 1 ||		\
		     (__GNUC_MINOR__ == 1 &&		\
		      __GNUC_PATCHLEVEL__ > 0)))
    
    const abi::__class_type_info* targetTI = 
      (const abi::__class_type_info *)( &(target_type));
    
    creaDebugMessage("info",7," * source   = "<<source_pointer<<std::endl);

    void* tmp = source_pointer;
    if (source_type.__do_upcast(targetTI,&tmp)) 
      {
	target_pointer = tmp;
      }
    else 
      {
	creaDebugMessage("info",7,
			 " * upcast failed : trying dynamic down cast"
			 <<std::endl);
	const abi::__class_type_info* sourceTI = 
	  (const abi::__class_type_info *)( &(source_type));
	
	
	target_pointer = abi::__dynamic_cast(source_pointer, 
					     sourceTI, 
					     targetTI, 
					     -1);   
      }
    
    creaDebugMessage("info",7," * target   = "<<target_pointer<<std::endl);
    
#else
    creaWarning("run_time_up_or_down_cast not impl. on Win : to do");
    // target_pointer = __RTDynamicCast(source_pointer, 0, source_type, target_type, 0);
#endif
    return target_pointer;
    
  }

}

